package com.ruoyi.web.controller.tool;

import java.util.HashMap;
import java.util.Map;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import us.codecraft.webmagic.Page;
import us.codecraft.webmagic.Site;
import us.codecraft.webmagic.Spider;
import us.codecraft.webmagic.processor.PageProcessor;

/**
 * 佐川急便物流查询控制器
 * 
 * @author ruoyi
 */
@Api("佐川急便物流查询接口")
@RestController
@RequestMapping("/tool/sagawa")
@Anonymous
public class SagawaExpressController extends BaseController implements PageProcessor {
    private static final Logger logger = LoggerFactory.getLogger(SagawaExpressController.class);

    // 站点配置
    private final Site site = Site.me()
            .setRetryTimes(3)
            .setSleepTime(1000)
            .setTimeOut(99999999)
            .setUserAgent("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36");

    // 查询结果
    private Map<String, Object> resultMap = new HashMap<>();
    @Override
    public void process(Page page) {
        try {
            String pageContent = page.getHtml().toString();
            
            // 检查是否有包裹数据未登记的信息
            boolean hasNoData = pageContent.contains("お荷物データが登録されておりません");
            
            if (hasNoData) {
                resultMap.put("status", "notFound");
                resultMap.put("message", "没有找到对应的包裹信息");
                return;
            }

            String trackingTableRegex = "<table[^>]*class=\"table_basic table_okurijo_detail2\"[^>]*>\\s*<tbody>\\s*(?:<tr>.*?<th[^>]*>荷物状況</th>.*?</tr>.*?<tr>.*?</tr>.*?)+\\s*</tbody>\\s*</table>";
            Pattern pattern = Pattern.compile(trackingTableRegex, Pattern.DOTALL);
            Matcher matcher = pattern.matcher(pageContent);
            
            if (matcher.find()) {
                String trackingTable = matcher.group(0);
                
                // 提取表格中的最后一行
                String rowRegex = "<tr>\\s*<td>\\s*([^<]*?)\\s*</td>\\s*<td>\\s*([^<]*?)\\s*</td>\\s*<td>\\s*([^<]*?)\\s*</td>\\s*</tr>";
                Pattern rowPattern = Pattern.compile(rowRegex, Pattern.DOTALL);
                Matcher rowMatcher = rowPattern.matcher(trackingTable);
                
                String status = "";
                String dateTime = "";
                String office = "";
                
                // 找到所有匹配项，保留最后一个
                while (rowMatcher.find()) {
                    status = rowMatcher.group(1).trim();
                    dateTime = rowMatcher.group(2).trim();
                    office = rowMatcher.group(3).trim();
                }
                
                if (!status.isEmpty()) {
                    Map<String, String> trackInfo = new HashMap<>();
                    trackInfo.put("status", status);
                    trackInfo.put("dateTime", dateTime);
                    trackInfo.put("office", office);
                    
                    resultMap.put("status", "success");
                    resultMap.put("trackInfo", trackInfo);
                } else {
                    resultMap.put("status", "noRecords");
                    resultMap.put("message", "没有物流记录");
                }
            } else {
                resultMap.put("status", "noTable");
                resultMap.put("message", "未找到物流跟踪表格");
            }
        } catch (Exception e) {
            logger.error("解析页面失败", e);
            resultMap.put("status", "error");
            resultMap.put("message", "解析页面失败: " + e.getMessage());
        }
    }

    @Override
    public Site getSite() {
        return site;
    }

    /**
     * 构建佐川急便查询URL
     */
    private String buildSagawaUrl(String trackingNumber) {
        return "https://k2k.sagawa-exp.co.jp/p/web/okurijosearch.do?okurijoNo=" + trackingNumber.trim();
    }

    /**
     * 查询佐川急便物流信息
     */
    @ApiOperation("查询佐川急便物流信息")
    @GetMapping("/tracking/{trackingNumber}")
    public R<Map<String, Object>> getTrackingInfo(@PathVariable("trackingNumber") String trackingNumber) {
        try {
            if (trackingNumber == null || trackingNumber.trim().isEmpty()) {
                return R.fail("运单号不能为空");
            }

            resultMap = new HashMap<>();
            String url = buildSagawaUrl(trackingNumber);
            
            try {
                Spider spider = Spider.create(this)
                        .addUrl(url)
                        .thread(1);
                spider.run();
            } catch (Exception e) {
                logger.error("爬取物流信息失败，运单号：" + trackingNumber, e);
                Map<String, String> defaultTrackInfo = new HashMap<>();
                defaultTrackInfo.put("status", "处理中");
                defaultTrackInfo.put("dateTime", "");
                defaultTrackInfo.put("office", "");
                
                resultMap.put("status", "success");
                resultMap.put("trackInfo", defaultTrackInfo);
            }
            
            return R.ok(resultMap);
        } catch (Exception e) {
            logger.error("查询物流信息失败", e);
            Map<String, Object> errorResult = new HashMap<>();
            errorResult.put("status", "error");
            errorResult.put("message", "查询物流信息失败: " + e.getMessage());
            return R.ok(errorResult);
        }
    }
} 