/**
 * Excel工具类 - 用于导入导出Excel文件
 */
import axios from 'axios'
import { saveAs } from 'file-saver'
import * as XLSX from 'xlsx'
import { getToken } from '@/utils/auth'

/**
 * 解析Excel文件
 * @param {String} url - Excel文件URL
 * @returns {Promise<Array>} - 解析后的数据数组
 */
export function parseExcel(url) {
  return new Promise((resolve, reject) => {
    axios({
      method: 'get',
      url: process.env.VUE_APP_BASE_API + url,
      responseType: 'arraybuffer',
      headers: { 'Authorization': 'Bearer ' + getToken() }
    }).then(response => {
      try {
        // 使用xlsx库解析Excel数据
        const data = new Uint8Array(response.data)
        const workbook = XLSX.read(data, { type: 'array' })
        const firstSheetName = workbook.SheetNames[0]
        const worksheet = workbook.Sheets[firstSheetName]
        const jsonData = XLSX.utils.sheet_to_json(worksheet, { header: 1 })
        resolve(jsonData)
      } catch (error) {
        reject('解析Excel失败: ' + error.message)
      }
    }).catch(error => {
      reject('下载Excel失败: ' + error.message)
    })
  })
}

/**
 * 从本地文件直接读取Excel数据
 * @param {File} file - 上传的Excel文件对象
 * @returns {Promise<Array>} - 解析后的数据数组
 */
export function parseExcelFile(file) {
  return new Promise((resolve, reject) => {
    try {
      const reader = new FileReader();
      reader.onload = (e) => {
        try {
          const data = new Uint8Array(e.target.result);
          const workbook = XLSX.read(data, { type: 'array' });
          const firstSheetName = workbook.SheetNames[0];
          const worksheet = workbook.Sheets[firstSheetName];
          const jsonData = XLSX.utils.sheet_to_json(worksheet, { header: 1 });
          resolve(jsonData);
        } catch (error) {
          reject('解析Excel文件失败: ' + error.message);
        }
      };
      reader.onerror = (error) => reject('读取文件失败: ' + error);
      reader.readAsArrayBuffer(file);
    } catch (error) {
      reject('处理文件失败: ' + error.message);
    }
  });
}

/**
 * 导出数据到Excel文件
 * @param {Array} data - 要导出的数据数组
 * @param {String} filename - 文件名(不含扩展名)
 * @param {String} ext - 文件扩展名，默认为.xlsx
 */
export function exportExcel(data, filename, ext = '.xlsx') {
  const worksheet = XLSX.utils.aoa_to_sheet(data)
  const workbook = XLSX.utils.book_new()
  XLSX.utils.book_append_sheet(workbook, worksheet, 'Sheet1')
  
  // 确定文件类型
  let bookType = 'xlsx';
  let mimeType = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';
  
  // 根据扩展名确定文件类型
  if (ext === '.xls') {
    bookType = 'xls';
    mimeType = 'application/vnd.ms-excel';
  } else if (ext === '.et') {
    // WPS表格格式，实际上仍使用xlsx格式但改变扩展名
    bookType = 'xlsx';
    mimeType = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';
  }
  
  // 生成Excel文件并下载
  const excelBuffer = XLSX.write(workbook, { bookType: bookType, type: 'array' })
  const blob = new Blob([excelBuffer], { type: mimeType })
  
  // 确保扩展名正确
  const fileExt = ext.startsWith('.') ? ext : `.${ext}`;
  saveAs(blob, filename + fileExt)
} 